﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class CookBook : MonoBehaviour {

	public bool DEBUG_SHOW_RECIPES = false;

	public int[][] CBInfo;
	public bool[][] CBReveal;

	public int difficulty;
	public int complexity;
	public int MAX_DIFFICULTY;
	public int MAX_COMPLEXITY;
	public bool exploded = false;

	public int dish = 0;
	public List<GameOrder> orders = new List<GameOrder> ();

	public Texture2D redX;
	public Texture2D bartex;
	public Texture2D bgTex;
	public Texture2D titlecard;

	public Texture2D[] dishes;
	public Texture2D[] components;

	public GUIStyle ticketStyle;
	public GUIStyle noStyle;
	public GUIStyle moneyStyle;
	public GUIStyle tutStyle;

	int SPACING_x = 50;
	int SPACING_y = 50;
	int SPACING_width = 50;
	int SPACING_height = 50;

	public float cooldown_order;
	public float cooldown_ingredient;
	int [] ingredient_quantity;
	float [] cooldown_restock;
	public int money;
	public int money_to_next;
	public int tipbonus;
	public int flavor;
	public int gamelevel;
	public string gamestate;
	public int tutorialstep;
	float diff_OrderCDMin;
	float diff_OrderCDMax;
	int diff_FireThreshold;
	int satisfaction;



	
	// Use this for initialization
	void Start () 
	{
		Debug.Log ("Starting Cookbook");

		Debug.Log ("Creating CBInfo");
		CBInfo = new int[MAX_DIFFICULTY][];
		Debug.Log ("Creating CBReveal");
		CBReveal = new bool[MAX_DIFFICULTY][];

		Debug.Log ("Creating Subarrays");
		for (int i=0; i<MAX_DIFFICULTY; ++i)
		{
			CBInfo[i] = new int[MAX_COMPLEXITY];
			CBReveal[i] = new bool[MAX_COMPLEXITY];
		}

		Debug.Log ("BuildBook()");
		BuildBook(difficulty, complexity);

		gamelevel = 0;
		money = 0;
		money_to_next = 500;
		cooldown_ingredient = 0.0f;
		cooldown_order = 0.0f;

		diff_OrderCDMin = 3.0f;
		diff_OrderCDMax = 7.0f;

		gamestate = "title";

		cooldown_restock = new float[MAX_COMPLEXITY];
		ingredient_quantity = new int[MAX_COMPLEXITY];

		for(int iloop = 0; iloop<MAX_COMPLEXITY; ++iloop)
		{
			Debug.Log ("InitLoop");
			cooldown_restock[iloop] = 0.0f;
			ingredient_quantity[iloop] = 10;
		}
		tipbonus = 0;
		flavor = 0;
		tutorialstep = 0;
	
	}
	
	// Update is called once per frame
	void Update () 
	{
		if(gamestate == "tutorial")
		{

			cooldown_ingredient -= Time.deltaTime;
			
			for(int iloop =0; iloop < MAX_COMPLEXITY; ++iloop)
			{
				if(cooldown_restock[iloop] > 0.0f)
				{
					cooldown_restock[iloop] -= Time.deltaTime;
					if(cooldown_restock[iloop] <= 0.0f)
					{
						ingredient_quantity[iloop] = 10;
					}
				}
			}

		}
		if(gamestate == "gameplay")
		{

			for(int iloop =0; iloop<orders.Count; ++iloop)
			{
				orders[iloop].UpdateTimer();
				if(orders[iloop].timeleft <= 0.0f)
				{
					money -= orders[iloop].value;
					orders.RemoveAt(iloop);
					--iloop;
					--satisfaction;
				}
			}

			cooldown_ingredient -= Time.deltaTime;
			cooldown_order -= Time.deltaTime;

			for(int iloop =0; iloop < MAX_COMPLEXITY; ++iloop)
			{
				if(cooldown_restock[iloop] > 0.0f)
				{
					cooldown_restock[iloop] -= Time.deltaTime;
					if(cooldown_restock[iloop] <= 0.0f)
					{
						ingredient_quantity[iloop] = 10;
					}
				}
			}

			if(cooldown_order <= 0.0f)
			{
				//add order with random cooldown based on difficulty
				orders.Add(new GameOrder(difficulty, difficulty * 3.0f));
				cooldown_order = Random.Range(diff_OrderCDMin, diff_OrderCDMax);
			}

		}

	
	}

	void OnGUI ()
	{
		GUI.DrawTexture(new Rect(0,0,800,600), bgTex);

		if(gamestate == "title")
		{
			//display splash

			GUI.DrawTexture(new Rect(0,0,800,600),titlecard);

			if(GUI.Button(new Rect(350, 300, 100, 150), "Start\nGame", ticketStyle))
			{
				gamestate = "tutorial";
			}

		}
		else if (gamestate == "tutorial")
		{
			if(tutorialstep >= 1) // show recipe grid
			{
				if(dish!= -1)
					GUI.DrawTexture(new Rect(500, 250, 200, 200), dishes[dish]);
				
				for(int jloop = 0; jloop<complexity; ++jloop)
				{
					GUI.DrawTexture(new Rect(100 + SPACING_x * jloop, 150, SPACING_width, SPACING_height), components[jloop]);
				}
				
				for(int iloop = 0; iloop < difficulty; ++iloop)
				{
					GUI.DrawTexture(new Rect(30, 200 + SPACING_y * iloop, SPACING_width, SPACING_height), dishes[iloop]);
					
					for(int jloop = 0; jloop < complexity; ++jloop)
					{
						if (CBReveal[iloop][jloop])
						{
							if(CBInfo[iloop][jloop] != -1)
							{
								GUI.DrawTexture(new Rect(100 + SPACING_x * jloop, 200 + SPACING_y * iloop, SPACING_width, SPACING_height), dishes[CBInfo[iloop][jloop]]);
							}
							else
							{
								GUI.DrawTexture(new Rect(100 + SPACING_x * jloop, 200 + SPACING_y * iloop, SPACING_width, SPACING_height), redX);
							}
						}
					}
				}
			}
			if(tutorialstep >= 2)
			{
				if(!exploded)
				{
					for(int iloop = 0; iloop<complexity; ++iloop)
					{
						if(GUI.Button(new Rect(380 + (iloop * 80), 450, 75, 75), components[iloop], noStyle))
						{
							if(ingredient_quantity[iloop] > 0 && cooldown_ingredient <= 0.0f)
							{
								++flavor;
								--ingredient_quantity[iloop];
								cooldown_ingredient = 0.5f;
								CBReveal[dish][iloop] = true;
								dish = CBInfo[dish][iloop];
								if(dish == -1)
									exploded = true;
							}
						}
						GUI.Label(new Rect(380 + (iloop * 80), 450, 75, 75), "x" + ingredient_quantity[iloop].ToString(), noStyle);

						if(cooldown_restock[iloop] <= 0.0f && ingredient_quantity[iloop] < 10)
						{
							if(GUI.Button(new Rect(380 + (iloop * 80), 530, 75, 50), "RESTOCK!"))
							{
								cooldown_restock[iloop] = 5.0f;
								ingredient_quantity[iloop] = 0;
							}
						}
						else if (cooldown_restock[iloop] > 0.0f)
						{
							
							GUI.color = Color.black;
							GUI.DrawTexture(new Rect(380 + (iloop * 80), 530, 75, 25), bartex);
							GUI.color = Color.gray;
							GUI.DrawTexture(new Rect(380 + (iloop * 80), 530, 75 * (cooldown_restock[iloop]/5.0f), 25), bartex);
							GUI.color = Color.white;
							GUI.Label(new Rect(380 + (iloop * 80), 530, 75, 25), "Restocking...");
							
						}
					}
				}
				else if (exploded)
				{
					if(GUI.Button(new Rect(380, 450, 400, 75), "It blew up! Click for new mush!"))
					{
						dish = 0;
						flavor = 0;
						exploded = false;
					}
				}
			}

			if(tutorialstep >= 6)
			{
				if(orders.Count > 0)
				{
					if(GUI.Button (new Rect(500, -5, 69,100), dishes[orders[0].requirement], ticketStyle))
					{
						if(dish == orders[0].requirement)
						{
							money += orders[0].value;
							money_to_next = 50;
							orders.RemoveAt (0);
							dish = 0;
							tutorialstep++;

						}
					}
				}
			}

			if(tutorialstep >= 7)
			{


				GUI.color = Color.black;
				GUI.DrawTexture(new Rect(475, 150, 200, 25), bartex);
				GUI.color = new Color(0.2f, 1.0f, 0.5f);
				GUI.Label (new Rect(475, 125, 200, 50), "Earned: $" + money.ToString(), moneyStyle);
				GUI.DrawTexture(new Rect(475, 150,(int)(200.0f * ((float)money/(float)money_to_next)), 25), bartex);
				GUI.color = Color.white;

			}


			if(tutorialstep >= 8)
			{

								
				GUI.color = Color.black;
				GUI.DrawTexture(new Rect(475, 200, 200, 25), bartex);
				if(satisfaction > 0)
					GUI.color = new Color(0.1f, 0.7f, 0.1f);
				else if(satisfaction < 0)
					GUI.color = new Color(0.7f, 0.1f, 0.1f);
				else 
					GUI.color = new Color(0.7f, 0.7f, 0.1f);

				GUI.Label (new Rect(475, 175, 200, 50), "Customer Satisfaction:", moneyStyle);
				GUI.DrawTexture(new Rect(475, 200,(int)(200.0f * (float)(satisfaction - diff_FireThreshold)/(float)(10-diff_FireThreshold)), 25), bartex);
				GUI.color = Color.white;
			}

			if(GUI.Button (new Rect(0, 550, 100, 50), "Skip Tutorial"))
			{
				gamelevel=0;
				SetStage ();
				gamestate = "gameplay";
			}
			
			switch(tutorialstep)
			{
			case 0: 
				difficulty = 3;
				complexity = 2;
				BuildBook(difficulty, complexity);
				
				if(GUI.Button(new Rect(250, 250, 300, 100), "Because capitalism, your boss has opened a restaurant that caters to the cravings of the beings from The Otherside.\n\n [click these bubbles to advance]", tutStyle)) 
				{
					tutorialstep++;
				}
				break;
			case 1:
				if(GUI.Button(new Rect(250, 250, 300, 100), "You, of course, know absolutely NOTHING about Otherside cuisine, or how it's cooked. But, customers are already lined up out the door, so you better learn quick!", tutStyle)) 
				{
					tutorialstep++;
				}
				break;
			case 2:
				if(GUI.Button(new Rect(450, 350, 300, 100), "Your boss instructed you to \"mix this stuff until it looks right\". ", tutStyle)) 
				{
					tutorialstep++;
				}
				break;
			case 3:
				if(GUI.Button(new Rect(450, 250, 300, 100), "Ingredients from Otherside morph dishes into other dishes, starting with this tasteless slime.", tutStyle)) 
				{
					tutorialstep++;
					for(int iloop = 0; iloop<difficulty; ++iloop)
					{
						for(int jloop = 0; jloop < complexity; ++jloop)
						{
							if(CBInfo[iloop][jloop] != -1)
								CBReveal[iloop][jloop] = true;
						}
					}
				}
				break;
			case 4:
				if(GUI.Button(new Rect(50, 350, 300, 100), "After every mix, the results will be recorded on your cheat sheet here. After each level, the sheet is erased because the combinations have changed!", tutStyle)) 
				{
					tutorialstep++;
					exploded = true;
					dish = -1;
					for(int iloop = 0; iloop<difficulty; ++iloop)
					{
						for(int jloop = 0; jloop < complexity; ++jloop)
						{
							CBReveal[iloop][jloop] = true;
						}
					}
				}
				break;
			case 5:
				if(GUI.Button(new Rect(250, 250, 300, 100), "Some mixes cause your creation to violently explode. You'll have to start your dish over if that happens.", tutStyle)) 
				{
					tutorialstep++;
					orders.Add(new GameOrder(difficulty, 10000.0f));
					exploded = false;
					dish = 0;
				}
				break;
			case 6:
				if(GUI.Button(new Rect(150, 0, 300, 100), "Orders will consistently come in through the wire. Make the dish by clicking the ingredients, and then click the order when it matches!", tutStyle)) 
				{
					++tutorialstep;
				}
				break;
			case 7:
				if(GUI.Button(new Rect(250, 250, 300, 100), "Each correct order will earn you some cash. You'll earn more money for completing dishes quickly. Your shift ends when you fill the income bar.", tutStyle)) 
				{
					tutorialstep++;
					satisfaction = 0;
					diff_FireThreshold = -5;
				}
				break;
			case 8:
				if(GUI.Button(new Rect(250, 250, 300, 100), "More ingredients added to a dish yields more flavor! Flavorful dishes will increase customer satisfaction!", tutStyle)) 
				{
					tutorialstep++;
				}
				break;
			case 9:
				if(GUI.Button(new Rect(250, 250, 300, 100), "Sending the wrong dish, or failing to fulfill an order will LOSE you money and customer satisfaction! If your customers become too dissatisfied, you'll be fired!", tutStyle)) 
				{
					tutorialstep++;
				}
				break;
				
			case 10:
				if(GUI.Button(new Rect(450, 350, 300, 100), "If you run out of ingredients, you'll have to restock them. This takes some time, during which you cannot use that ingredient!", tutStyle)) 
				{
					tutorialstep++;
				}
				break;
			case 11:
				if(GUI.Button(new Rect(450, 350, 300, 100), "Ready to jump in? Good luck!", tutStyle)) 
				{
					gamelevel=0;
					SetStage ();
					gamestate = "gameplay";
				}
				break;

			default:
				break;
			}

		}
		else if(gamestate == "gameplay")
		{


			
			GUI.color = Color.black;
			GUI.DrawTexture(new Rect(475, 150, 200, 25), bartex);
			GUI.color = new Color(0.2f, 1.0f, 0.5f);
			GUI.Label (new Rect(475, 125, 200, 50), "Income: $" + money.ToString(), moneyStyle);
			GUI.DrawTexture(new Rect(475, 150,(int)(200.0f * ((float)money/(float)money_to_next)), 25), bartex);
			GUI.color = Color.white;



			GUI.Label (new Rect(475, 175, 200, 50), "Customer Satisfaction:" + satisfaction.ToString(), moneyStyle);
			
			GUI.color = Color.black;
			GUI.DrawTexture(new Rect(475, 200, 200, 25), bartex);
			if(satisfaction > 0)
				GUI.color = new Color(0.1f, 0.7f, 0.1f);
			else if(satisfaction < 0)
				GUI.color = new Color(0.7f, 0.1f, 0.1f);
			else 
				GUI.color = new Color(0.7f, 0.7f, 0.1f);

			GUI.DrawTexture(new Rect(475, 200,(int)(200.0f * (float)(satisfaction - diff_FireThreshold)/(float)(10-diff_FireThreshold)), 25), bartex);
			GUI.color = Color.white;


			if(dish!= -1)
				GUI.DrawTexture(new Rect(500, 250, 200, 200), dishes[dish]);

			for(int jloop = 0; jloop<complexity; ++jloop)
			{
				GUI.DrawTexture(new Rect(100 + SPACING_x * jloop, 150, SPACING_width, SPACING_height), components[jloop]);
			}

			for(int iloop = 0; iloop < difficulty; ++iloop)
			{
				GUI.DrawTexture(new Rect(30, 200 + SPACING_y * iloop, SPACING_width, SPACING_height), dishes[iloop]);

				for(int jloop = 0; jloop < complexity; ++jloop)
				{
					if (CBReveal[iloop][jloop])
					{
						if(CBInfo[iloop][jloop] != -1)
						{
							GUI.DrawTexture(new Rect(100 + SPACING_x * jloop, 200 + SPACING_y * iloop, SPACING_width, SPACING_height), dishes[CBInfo[iloop][jloop]]);
						}
						else
						{
							GUI.DrawTexture(new Rect(100 + SPACING_x * jloop, 200 + SPACING_y * iloop, SPACING_width, SPACING_height), redX);
						}
					}
				}
			}

			/*
			if(GUI.Button(new Rect(500, 100, 200, 50), "DEBUG*REROLL"))
			{
				BuildBook(difficulty, complexity);
			}

			if(GUI.Button(new Rect(500, 175, 200, 50), "DEBUG*ADDORDER"))
			{
				orders.Add(new GameOrder(difficulty, difficulty * 3.0f));
			}
			*/

			if(cooldown_ingredient > 0.0f)
			{

				GUI.color = Color.black;
				GUI.DrawTexture(new Rect(550, 350, 100, 25), bartex);
				GUI.color = Color.gray;
				GUI.DrawTexture(new Rect(550, 350, 100 * (cooldown_ingredient/0.5f), 25), bartex);
				GUI.color = Color.white;
				GUI.Label(new Rect(550, 350, 100, 25), "Mixing...");

			}

			if(!exploded)
			{

				for(int iloop = 0; iloop<complexity; ++iloop)
				{
					if(GUI.Button(new Rect(380 + (iloop * 80), 450, 75, 75), components[iloop], noStyle))
					{
						if(cooldown_ingredient <= 0.0f && cooldown_restock[iloop] <= 0.0f && ingredient_quantity[iloop] > 0)
						{
							++flavor;
							--ingredient_quantity[iloop];
							CBReveal[dish][iloop] = true;
							cooldown_ingredient = 0.5f;
							dish = CBInfo[dish][iloop];
							if(dish == -1)
								exploded = true;
						}
					}
					GUI.Label(new Rect(380 + (iloop * 80), 450, 75, 75), "x" + ingredient_quantity[iloop].ToString(), noStyle);

					if(cooldown_restock[iloop] <= 0.0f && ingredient_quantity[iloop] < 10)
					{
						if(GUI.Button(new Rect(380 + (iloop * 80), 530, 75, 50), "RESTOCK!"))
						{
							cooldown_restock[iloop] = 5.0f;
							ingredient_quantity[iloop] = 0;
						}
					}
					else if (cooldown_restock[iloop] > 0.0f)
					{

						GUI.color = Color.black;
						GUI.DrawTexture(new Rect(380 + (iloop * 80), 530, 75, 25), bartex);
						GUI.color = Color.gray;
						GUI.DrawTexture(new Rect(380 + (iloop * 80), 530, 75 * (cooldown_restock[iloop]/5.0f), 25), bartex);
						GUI.color = Color.white;
						GUI.Label(new Rect(380 + (iloop * 80), 530, 75, 25), "Restocking...");

					}
				}

			}
			else if (exploded)
			{
				if(GUI.Button(new Rect(380, 450, 400, 75), "It blew up! Click for new mush!"))
				{
					dish = 0;
					flavor = 0;
					exploded = false;
				}
			}


			// DISPLAY ORDERS

			for(int iloop =0; iloop<orders.Count; ++iloop)
			{
				if(GUI.Button (new Rect( -100 + (900 * (orders[iloop].timeleft/(difficulty * 3.0f))), -5, 69,100), dishes[orders[iloop].requirement], ticketStyle))
				{
					if(dish == orders[iloop].requirement)
					{
						money += orders[iloop].value;

						tipbonus = 0;
						if(orders[iloop].timeleft >  0.3f * difficulty * complexity)
							tipbonus += (int)(orders[iloop].value * 0.2f);

						if(flavor >= 4)
						{
							satisfaction += (flavor - 3);
							if(satisfaction > 10)
								satisfaction = 10;
						}

						flavor = 0;
						money += tipbonus;
						orders.RemoveAt (iloop);
						--iloop;
						dish = 0;
					}
					else
					{
						money -= (orders[iloop].value / 2);
						orders.RemoveAt (iloop);
						--iloop;
						satisfaction -= 2;
						dish = 0;
					}
				}
			}

			if(money >= money_to_next)
			{
				gamestate = "nextlevel";
			}
			if(satisfaction <= diff_FireThreshold)
			{
				gamestate = "gameover";
			}

		}
		else if (gamestate == "nextlevel")
		{
			if(GUI.Button(new Rect(250, 250, 300, 100), "Good work! You've made it through your shift! But before you know it, another day begins...", tutStyle)) 
			{
				++gamelevel;
				SetStage ();
				gamestate = "gameplay";
			}

		}
		else if (gamestate == "gameover")
		{
			if(GUI.Button(new Rect(250, 250, 300, 100), "Your inability to cook food you know nothing about has gotten you fired!\n\nClick here to try again.", tutStyle)) 
			{
				SetStage ();
				gamestate = "gameplay";
			}
		}

	}

	void BuildBook(int outcomes, int options)
	{

		if(outcomes > MAX_DIFFICULTY || options > MAX_COMPLEXITY || options == 1 || outcomes == 1)
		{
			Debug.Log ("BUILDBOOK PARAMETERS BAD!");
			return;
		}
		//clear the book (set everything to explode)
		for(int iloop = 0; iloop<MAX_DIFFICULTY;++iloop)
		{
			for(int jloop = 0; jloop < MAX_COMPLEXITY; ++jloop)
			{
				CBInfo[iloop][jloop] = -1;
				CBReveal[iloop][jloop] = DEBUG_SHOW_RECIPES; // CHANGE TO FALSE WHEN NOT DEBUGGING
			}
		}

		// make sure every dish has at least 2 connections
		for(int repeatloop = 0; repeatloop < Mathf.CeilToInt((float)options/2.0f); ++repeatloop)
		{
			for(int iloop = 1; iloop<outcomes; ++iloop)
			{
				bool cb_success = false;

				while (cb_success != true)
				{
					cb_success = false;

					int holdx = Random.Range(1,outcomes);
					int holdy = Random.Range(0,options);

					// if the slot is an explosion, and it's not the same dish creating itself
					if((CBInfo[holdx][holdy] == -1) && (holdx != iloop))
					{
						CBInfo[holdx][holdy] = iloop;
						cb_success = true;
					}
					else
					{
						cb_success = false;
					}
				}
			}
		}

		//make sure the 0 dish has 3 connections
		for(int jloop = 0; jloop < options; ++jloop)
		{
			if((CBInfo[0][jloop] == -1))
			{
				CBInfo[0][jloop] = Random.Range (1, outcomes);
			}
		}
	
	}

	public void SetStage()
	{
		switch(gamelevel)
		{
		case 0:
			difficulty = 3;
			complexity = 2;
			money_to_next = 200;
			diff_OrderCDMin = 3.0f;
			diff_OrderCDMax = 5.0f;
			diff_FireThreshold = -5;
			break;
		case 1:
			difficulty = 4;
			complexity = 3;
			money_to_next = 300;
			diff_OrderCDMin = 2.0f;
			diff_OrderCDMax = 5.0f;
			diff_FireThreshold = -5;
			break;
		case 2:
			difficulty = 5;
			complexity = 3;
			money_to_next = 400;
			diff_OrderCDMin = 2.0f;
			diff_OrderCDMax = 5.0f;
			diff_FireThreshold = -5;
			break;
		case 3:
			difficulty = 6;
			complexity = 4;
			money_to_next = 500;
			diff_OrderCDMin = 2.0f;
			diff_OrderCDMax = 4.0f;
			diff_FireThreshold = -5;
			break;
		case 4:
			difficulty = 7;
			complexity = 4;
			money_to_next = 600;
			diff_OrderCDMin = 2.0f;
			diff_OrderCDMax = 4.0f;
			diff_FireThreshold = -5;
			break;
		case 5:
			difficulty = 7;
			complexity = 5;
			money_to_next = 700;
			diff_OrderCDMin = 2.0f;
			diff_OrderCDMax = 4.0f;
			diff_FireThreshold = -5;
			break;
		default:
			difficulty = 7;
			complexity = 5;
			money_to_next = 900 + (100 * (gamelevel - 6));
			diff_OrderCDMin = 2.0f;
			diff_OrderCDMax = 3.0f;
			diff_FireThreshold = -5;
			break;
		}

		for(int iloop = 0; iloop < complexity; ++iloop)
		{
			ingredient_quantity[iloop] = 10;
			cooldown_restock[iloop] = 0.0f;
			cooldown_ingredient = 0.0f;
		}
		orders.Clear ();
		exploded = false;
		BuildBook (difficulty, complexity);
		dish = 0;
		satisfaction = 0;
		money = 0;
	}
}
